/*
Class-name: DrawPanel.java, part of the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

package modeller;

import interfaces.CadObject;
import interfaces.Layer;
import interfaces.Vertex;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Rectangle2D;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import modeller.draw.HotSpot;
import modeller.draw.Raster;
import static modeller.draw.StaticDrawing.*;
import modeller.draw.StrechingRectangle;
import modeller.popups.ViewPopup;

/**
 * Class for the drawingdesktop in front-, side- and topview of the 3d-object 
 * @author bettray
 *
 */
public class DrawPanel extends JPanel implements MouseListener, ComponentListener, MouseMotionListener
{
	private int panelID;
	private int pressedButton;
	private int x1, y1, z1, x2, y2, z2;
	private double panelClipX = 0, panelClipY = 0, panelClipWidth = 0, panelClipHeight = 0;
	private int hotSpotsSet = 0;
	private double scale = 1.0d, swapScale;
//	private double translateX = 0, translateY = 0, translateZ = 0;
	private boolean fetchButtonReleased = false, fetchButtonclicked = false;

	private Color panelBackground = Color.LIGHT_GRAY;
	private Shape swapShape;
	
	private ViewPopup viewPopup;
	private ModellerStack modellerStack;
	private StatusPanel statusPanel;
	private Raster raster;
	
	private JScrollPane scrollPane;
	private CadObject swapCadObject;
	private HotSpot hotSpot;
	
	/**
	 * 
	 * @param statusPanel
	 * @param modellerStack
	 * @param panelID
	 */
	public DrawPanel(StatusPanel statusPanel, ModellerStack modellerStack, int panelID)
	{
		this.statusPanel = statusPanel;
		this.modellerStack = modellerStack;
		this.panelID = panelID;
		
		raster = new Raster(modellerStack);
		
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addComponentListener(this);
		this.setBackground(Color.LIGHT_GRAY);
		this.add(new JLabel(this.getClass().getSimpleName()));
		
		refreshDimension();
	}
	
	public void paint(Graphics g)
	{
		super.paintComponent(g);
		
//		System.out.print("panelID "+panelID+": ");
		
		Graphics2D g2 = (Graphics2D) g;
		
		swapScale = scale;
		
		scale *= modellerStack.pixelPerMm;
		
		// Painting only the visible cliparea
//		g2.clipRect(0, 0, (int)panelClipWidth, (int)panelClipHeight);
		g2.clipRect(-(int)panelClipX, -(int)panelClipY, (int)panelClipWidth, (int)panelClipHeight);
//		 Painting only the visible cliparea - END
		
		// Setting the scaling
		g2.scale(scale, scale);
		// Setting the scaling - END

		// Painting the background
//		g2.setColor(panelBackground); // Color.LIGHT_GRAY);
//		g2.fill(raster.generateBackground());
		// Painting the background - END
		
		// Painting the raster
		g2.setStroke(new BasicStroke(1f / (float)scale));
		if(panelID==0) {
			g2.setColor(raster.getColor());
			g2.draw(raster.generateRaster(modellerStack.boundaryWidth, modellerStack.boundaryHeight));
			g2.setColor(Color.BLACK);
			g2.draw(new Rectangle2D.Double(0, 0, modellerStack.boundaryWidth, modellerStack.boundaryHeight));
		} else if(panelID==1) {
			g2.setColor(raster.getColor());
			g2.draw(raster.generateRaster(modellerStack.boundaryDepth, modellerStack.boundaryHeight));
			g2.setColor(Color.BLACK);
			g2.draw(new Rectangle2D.Double(0, 0, modellerStack.boundaryDepth, modellerStack.boundaryHeight));
		} else {
			g2.setColor(raster.getColor());
			g2.draw(raster.generateRaster(modellerStack.boundaryWidth, modellerStack.boundaryDepth));
			g2.setColor(Color.BLACK);
			g2.draw(new Rectangle2D.Double(0, 0, modellerStack.boundaryWidth, modellerStack.boundaryDepth));
		}
		// Painting the raster - END
		
		// Painting the generated objects
		if(modellerStack.cadObjects.size()>0) {
			if(modellerStack.objectMoving) moveObject();
			
			for(int i=0; i < modellerStack.cadObjects.size(); i++) {
				swapCadObject = modellerStack.cadObjects.get(i);
				if(swapCadObject.isVisible()) {
//					StaticDrawing.paintingObject(g2, swapFemObject, i, panelID, scale, (modellerStack.selectedObject==i));
					paintingObject(g2, swapCadObject, i);
				}
			}
		}
		// Painting the generated objects - END
		
		// painting the cursor-cross
		g2.scale(1d / scale, 1d / scale);
		g2.setStroke(FINE_DOTTED_STROKE);
		g2.setColor(Color.WHITE);
		g2.draw(drawCross(modellerStack.xMoved, modellerStack.yMoved, modellerStack.zMoved, panelID, getWidth(), getHeight()));
		if(modellerStack.mousePressed)
			g2.draw(drawCross(modellerStack.xDragged, modellerStack.yDragged, modellerStack.zDragged, panelID, getWidth(), getHeight()));
		// painting the cursor-cross - END
		
		// Streching the Object
//		if(modellerStack.cadObjects.size()>0)
//			if(modellerStack.objectStretching) stretchObject(g2);
		// Streching the Object - END
		
		// painting the helpline
		if(modellerStack.stillBuilding && fetchButtonReleased) {	// Painting dooted line, if in build-mode
//			g2.setStroke(FINE_DOTTED_STROKE);
			g2.setColor(Color.WHITE);
			g2.draw(drawPaintLine(modellerStack.xPressed, modellerStack.yPressed, modellerStack.zPressed, modellerStack.xMoved, modellerStack.yMoved, modellerStack.zMoved, panelID, getWidth(), getHeight()));
		}
		else if(modellerStack.mousePressed && !modellerStack.objectMoving && panelID==modellerStack.currentDrawPanelID) {	// painting normal line if not still building
			g2.setStroke(BASICSTROKE);
			g2.setColor(Color.WHITE);
			g2.draw(drawPaintLine(modellerStack.xPressed, modellerStack.yPressed, modellerStack.zPressed, modellerStack.xDragged, modellerStack.yDragged, modellerStack.zDragged, panelID, getWidth(), getHeight()));
		}
		// painting the helpline - END
		
		// Setting the marking-hotspots and painting current selected, new CadObject
		if(modellerStack.stillBuilding && fetchButtonReleased && modellerStack.currentCadObject!=null) {	// (modellerStack.currentObjectPanelID==panelID)

			// Setting the coordinates by panelID
			if(panelID==0) {
				x1 = modellerStack.xPressed; y1 = modellerStack.yPressed;  x2 = modellerStack.xMoved; y2 = modellerStack.yMoved; z1 = 0; z2 = 0;
			} else if(panelID==1) {
				y1 = modellerStack.yPressed; z1 = modellerStack.zPressed; y2 = modellerStack.yMoved; z2 = modellerStack.zMoved; x1 = 0;  x2 = 0;
			} else {
				x1 = modellerStack.xPressed; z1 = modellerStack.zPressed; x2 = modellerStack.xMoved; z2 = modellerStack.zMoved; y1 = 0; y2 = 0;
			}
			
			// Setting the object-points for objects with number of points > 0
			if(modellerStack.currentCadObject!=null && modellerStack.currentCadObject.getNumberOfPoints()>0) {
				modellerStack.currentCadObject.setObjectPoints(hotSpotsSet, x1, y1, z1, x2, y2, z2, scale);
				modellerStack.currentCadObject.finalizeObject(panelID);

				// Setting the marking-spots for objects with number of points > 0
				if(modellerStack.mouseReleased && hotSpotsSet<modellerStack.currentCadObject.getNumberOfPoints()) {
					modellerStack.hotSpots.add(new HotSpot(modellerStack.xPressed, modellerStack.yPressed, modellerStack.zPressed, scale));
					hotSpotsSet++;
				}
			}
			else {
				// Setting the object-points for objects with unknown number of points (means: member 'numberOfPoints' = 0
				modellerStack.currentCadObject.setObjectPoints(hotSpotsSet, x1, y1, z1, x2, y2, z2, scale);
				modellerStack.currentCadObject.finalizeObject(panelID);
				
				// Setting the marking-spots for objects with unknown number of points
				if(fetchButtonclicked) {
					fetchButtonclicked = false;
					modellerStack.hotSpots.add(new HotSpot(modellerStack.xPressed, modellerStack.yPressed, modellerStack.zPressed, scale));
					hotSpotsSet++;
				}
			}
			// Setting the id of the panel, where the object is currently in build process 
			modellerStack.currentCadObject.setBuiltInView(panelID);
			
			// painting current selected, new CadObject
			g2.setStroke(BASICSTROKE);
			g2.setColor(modellerStack.currentCadObject.getLineColor());
			g2.draw(modellerStack.currentCadObject.getShape(panelID, scale));

			// Storing the N-Points-Objects  
			if(modellerStack.currentCadObject!=null && (pressedButton==MouseEvent.BUTTON3 || hotSpotsSet==modellerStack.currentCadObject.getNumberOfPoints())) {
				storeCadObject();
				modellerStack.hotSpots.clear();
			}
		}
		// Setting the marking-hotspots and painting current selected, new CadObject - END

		// painting all existing hotSpots
		if(modellerStack.hotSpots.size()>0) {
			g2.setColor(Color.BLACK);
			g2.setStroke(BASICSTROKE);
			for(int i=0; i<modellerStack.hotSpots.size(); i++) {
				hotSpot = modellerStack.hotSpots.get(i);
				g2.setColor(Color.BLACK);
				g2.draw(drawMarkingSpot(hotSpot.x, hotSpot.y, hotSpot.z, scale, 5, panelID));
			}
		}
		// painting all existing hotSpots - END
		
		scale = swapScale;
	}
		
	/**
	 * Moving a the selected CAD-Object in x-y-z direction 
	 */
	private void moveObject()
	{
		// Setting up the buffers-variables
		double x = 0, y = 0;
		double x1 = 0, y1 = 0, z1 = 0;
		double x2 = 0, y2 = 0, z2 = 0;
		double dx = 0, dy = 0, dz = 0;
		
		// Setting up the buffers-objects
		CadObject cadObject;
		Shape shape;
		Vertex vertex;
		
		// Only move the object, if the mouse is in drag-mode
		if(modellerStack.mouseDragged && modellerStack.selectedObject>=0) {
			cadObject = modellerStack.cadObjects.get(modellerStack.selectedObject);
			cadObject.setRendered2D(false);
			
			shape = cadObject.getShape(panelID);
//			cadObject.setRendered2D(false);
			
			if(panelID==0) {		// xy-view (front)
				x = modellerStack.xDragged / scale; y = modellerStack.yDragged / scale;
			}
			else if(panelID==1) {	// zy-view (side)
				x = modellerStack.zDragged / scale; y = modellerStack.yDragged / scale;
			}
			else {					// xz-view (top)
				x = modellerStack.xDragged / scale; y = modellerStack.zDragged / scale;
			}

			// Only move the object, if the mouse-position is in the shape-area of the object
			if((shape.getBounds().contains(x, y)) && modellerStack.mouseDragged) {	//  || shape.getBounds().contains(x + 1, y + 1)
				for(int i=0; i<cadObject.getVertexList().size(); i++) {
					
					// Getting the click-point and the drag-point
					x1 = modellerStack.xPressed; y1 = modellerStack.yPressed; z1 = modellerStack.zPressed;
					x2 = modellerStack.xDragged; y2 = modellerStack.yDragged; z2 = modellerStack.zDragged;
					
					// solving the difference between clicking and draging
//					dx = (x2 - x1); dy = (y2 - y1); dz = (z2 - z1);
					dx = (x2 - x1) / scale; dy = (y2 - y1) / scale; dz = (z2 - z1) / scale;
					
					// updating all vertex-points
					vertex = cadObject.getVertexList().get(i);
					vertex.setX(vertex.getX() + dx);
					vertex.setY(vertex.getY() + dy);
					vertex.setZ(vertex.getZ() + dz);
				}
				
				// copying the dragged-point to the click-point
				modellerStack.xPressed = modellerStack.xDragged;
				modellerStack.yPressed = modellerStack.yDragged;
				modellerStack.zPressed = modellerStack.zDragged;
			}
		}
	}

	/**
	 * Paints the FemObject, that is selected
	 * @param g2
	 */
	private void paintingObject(Graphics2D g2, CadObject swapCadObject, int selectedObjectIndex)
	{
		swapShape = swapCadObject.getShape(panelID);
		
//		double translX = 0, translY = 0;
		
		// Setting the translation, if it's necessary
//		if(translateX < 0 || translateY < 0 || translateZ < 0) {
//			if(panelID==0) {
//				translX = translateX;
//				translY = translateY;
//			} else if(panelID==1) {
//				translX = translateZ;
//				translY = translateY;
//			} else {
//				translX = translateX;
//				translY = translateZ;
//			}
//			g2.translate(translX, translY);
//		}
		// Setting the translation, if it's necessary - END
		
		if(swapCadObject.isFilled()) {					// if object is filled, paint filled oject
			if(swapCadObject.getTransparency()<1d && swapCadObject.getTransparency()>0d)
				g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, (float)swapCadObject.getTransparency()));
			if(swapCadObject.getTransparency()>0d) {
				g2.setColor(swapCadObject.getFillColor());
				g2.fill(swapShape);
			}
		}
		
		// Resetting the transparency
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1f));

		// painting dashed stroke if object is selected
		if(modellerStack.selectedObject==selectedObjectIndex) {
			g2.setStroke((GET_DASHED_STROKE(scale)));
			g2.setColor(Color.DARK_GRAY);
			g2.draw(drawMarkingBox(swapShape));
			g2.setStroke((GET_BASICSTROKE(scale)));

			// painting the filling of the marking-spots 
			g2.setColor(Color.CYAN);
			g2.fill(drawMarkingSpots(swapShape, 1d / scale, 3));

			// painting the border of the marking-spots 
			g2.setColor(Color.BLACK);
			g2.draw(drawMarkingSpots(swapShape, 1d / scale, 3));
			g2.setStroke(new BasicStroke((float)(swapCadObject.getLineWidth())));
		}
		
		// painting the outline of the object
		g2.setColor(swapCadObject.getLineColor());
		g2.draw(swapShape);
		
		// Resetting the translation, if it's necessary
//		if(translateX < 0 || translateY < 0 || translateZ < 0)
//			g2.translate(-translX, -translY);
	}

	private void stretchObject(Graphics2D g2d)
	{
		CadObject swapObject;
		Shape shape, swapShape;
		double dw, dh;

		swapObject = modellerStack.cadObjects.get(modellerStack.selectedObject);
		shape = swapObject.getShape(panelID);

//		Vertex vertex = swapObject.getVertexList().get(0);
		
//		System.out.println("Stretch-It");
		
		g2d.setStroke(BASICSTROKE);
		g2d.setColor(Color.BLACK);
		
		if(panelID==0) {
			x2 = modellerStack.xDragged; y2 = modellerStack.yDragged;
		} 
		else if(panelID==1) {
			x2 = modellerStack.zDragged; y2 = modellerStack.yDragged;
		} 
		else {
			x2 = modellerStack.xDragged; y2 = modellerStack.zDragged;
		}

		if(modellerStack.mouseDragged && modellerStack.selectedObject>=0) {
			
			if(x2>=shape.getBounds().getMinX() && y2 >= shape.getBounds().getMinY()) {
				g2d.setStroke(BASICSTROKE);
				g2d.setColor(Color.BLACK);
				g2d.draw(StrechingRectangle.getRectangle(shape.getBounds().getMinX(), shape.getBounds().getMinY(), x2, y2, panelID, scale));
			}
		}
		if(modellerStack.mouseReleased) {
			swapShape = StrechingRectangle.getRectangle(shape.getBounds().getMinX(), shape.getBounds().getMinY(), x2, y2, panelID, scale);

			dw = (double)swapShape.getBounds().getWidth() / (double)shape.getBounds().getWidth();
			dh = (double)swapShape.getBounds().getHeight() / (double)shape.getBounds().getHeight();
				
			System.out.println("(Stretch & released) dw: "+dw+" - dh: "+dh);
			
			StrechingRectangle.buildNewSize(swapObject.getVertexList(), panelID, dw, dh);	// modellerStack.cadObjects.get(modellerStack.selectedObject)
			
//			System.out.println("shape    : "+shape.getBounds().getWidth()+" - "+shape.getBounds().getHeight());
//			System.out.println("swapShape: "+swapShape.getBounds().getWidth()+" - "+swapShape.getBounds().getHeight());
		}
	}
	
	/*
	 * Snapping the objects by collision AND mouse-click
	 */
	private void selectObject()
	{
		double x, y;
		CadObject cadObject;
		
		if(panelID==0) {
			x = modellerStack.xMoved / scale; y = modellerStack.yMoved / scale;
		} else if(panelID==1) {
			x = modellerStack.zMoved / scale; y = modellerStack.yMoved / scale;
		} else  {
			x = modellerStack.xMoved / scale; y = modellerStack.zMoved / scale;
		}
			
		for(int i=0; i<modellerStack.cadObjects.size(); i++) {
			cadObject = modellerStack.cadObjects.get(i);
			
			if(cadObject.isVisible()) {
				if(cadObject.getLineTyp()==GEOMETRY_2D_LINE) {
					if(cadObject.getShape(panelID, scale).getBounds().contains(x, y)) {
						if(modellerStack.selectedObject!=i) {
							modellerStack.selectedObject = i;
							modellerStack.objectsProperties.updateObjectList(cadObject, true);
							
							// updating the layer-panel
							modellerStack.layerPanel[0].updateLayer(cadObject.getLayerID());
							modellerStack.layerPanel[1].updateLayer(cadObject.getLayerID());
							break;
						}
					}
				}
				else if(cadObject.getShape(panelID, scale).contains(x, y)) {
					if(modellerStack.selectedObject!=i) {
						modellerStack.selectedObject = i;

						modellerStack.objectsProperties.updateObjectList(cadObject, true);

						// updating the layer-panel
						modellerStack.layerPanel[0].updateLayer(cadObject.getLayerID());
						modellerStack.layerPanel[1].updateLayer(cadObject.getLayerID());
						break;
					}
				}
			}
		}
	}
	
	/**
	 * Refreshing the dimensions of the drawPanel, for fitting the ScrollPane
	 * @param factor
	 */
	public void refreshDimension()	// refreshDimension(int factor)
	{
		// System.out.println("refreshDimension");
		
		this.setVisible(false);
		this.setPreferredSize(new Dimension((int)(scale * modellerStack.boundaryWidth), (int)(scale * modellerStack.boundaryHeight)));
//		this.setPreferredSize(new Dimension((int)(scale * raster.getRasterWidth()), (int)(scale * raster.getRasterHeight())));
		this.setVisible(true);
	}
	
	/**
	 * Repainting the other 2 drawPanels
	 *
	 */
	private void refreshOtherDrawpanels()
	{
		// System.out.println("refreshOtherDrawpanels");
		
		if(panelID==0) {
			modellerStack.drawPanels[1].refreshDimension();
			modellerStack.drawPanels[2].refreshDimension();
		} else if(panelID==1) {
			modellerStack.drawPanels[0].refreshDimension();
			modellerStack.drawPanels[2].refreshDimension();
		} else if(panelID==2) {
			modellerStack.drawPanels[0].refreshDimension();
			modellerStack.drawPanels[1].refreshDimension();
		} 		
	}
	
	/**
	 * Storing the just builded FemOject-instance
	 *
	 */
	private void storeCadObject()
	{
//		System.out.println("storeFemObject");
		Layer layer = modellerStack.layers.get(modellerStack.currentCadObject.getLayerID());

		System.out.println("ExtrStart: "+layer.getExtrusionStart()+" - ExtrStop"+layer.getExtrusionStop());
		
		modellerStack.currentCadObject.setObjectId(modellerStack.cadObjects.size());
		modellerStack.currentCadObject.setExtrudingDirection(layer.getExtrudingDirection());
		modellerStack.currentCadObject.setExtrusionStart(layer.getExtrusionStart());
		modellerStack.currentCadObject.setExtrusionStop(layer.getExtrusionStop());
		modellerStack.currentCadObject.setRendered2D(false);
		modellerStack.currentCadObject.setInBuildProcess(false);
		
		modellerStack.cadObjects.add(modellerStack.currentCadObject);
//		repaint();
		modellerStack.repaintDrawPanels();
		
		modellerStack.stillBuilding = false;
		fetchButtonReleased = false;
		modellerStack.pointFixed = false;
		modellerStack.currentCadObject = null;
		hotSpotsSet = 0;
		modellerStack.objects.updateObjectList();
		modellerStack.swapButton.setBackground(this.getBackground());
	}
	
	/**
	 * Getting the current scaling-factor 
	 * @return
	 */
	public double getScale() {
		return scale;
	}

	/**
	 * Setting the new scaling-factor and controlling the min & max value of it
	 * @param scale
	 */
	public void setScale(double scale) {
		this.scale = scale;
		
		if(this.scale>10f)
			this.scale = 10f;
		
		if(this.scale<0.05f)
			this.scale=0.05f;
	}		 

	private void updateInfoPanel()
	{
		statusPanel.updatePanel();

	}

	// Eventhandlers for the mouse-movement above the drawPanel 
	public void mouseClicked(MouseEvent e)
	{
//		System.out.println("mouseClicked");
		fetchButtonclicked = true;
	}

	public void mouseEntered(MouseEvent e)
	{
//		System.out.println("mouseEntered");
	}

	public void mouseExited(MouseEvent e)
	{
//		System.out.println("mouseExited");
	}

	public void mousePressed(MouseEvent e)
	{
//		System.out.println("mousePressed:"+e.getButton());
		
		pressedButton=e.getButton();

		modellerStack.setMouseFlags(true, false, false, false, false, false);

			if(modellerStack.selectionToolButtonPressed && e.getButton()==MouseEvent.BUTTON1)
				selectObject();

		// Setting the xyz-positions of the mouse-cursor
		if(panelID==0) {
			modellerStack.xPressed = e.getX();
			modellerStack.yPressed = e.getY();
		} else if(panelID==1) {
			modellerStack.zPressed = e.getX();
			modellerStack.yPressed = e.getY();
		} else if(panelID==2) {
			modellerStack.xPressed = e.getX();
			modellerStack.zPressed = e.getY();
		}

		// Showing the Popup for the view-menu
		if(e.getButton()==MouseEvent.BUTTON2) {
//			System.out.println("BUTTON2");
			viewPopup.show(e.getComponent(), e.getX(), e.getY());
		}

		modellerStack.pointFixed = false;
		updateInfoPanel();
		repaint();
	}

	public void mouseReleased(MouseEvent e)
	{
//		System.out.println("mouseReleased");
		
		modellerStack.setMouseFlags(false, false, false, true, false, false);
		fetchButtonReleased = true;
		fetchButtonclicked = false;

		
		// Setting the xyz-positions of the mouse-cursor
		if(panelID==0) {
			modellerStack.xReleased = e.getX();
			modellerStack.yReleased = e.getY();
		} else if(panelID==1) {
			modellerStack.zReleased = e.getX();
			modellerStack.yReleased = e.getY();
		} else if(panelID==2) {
			modellerStack.xReleased = e.getX();
			modellerStack.zReleased = e.getY();
		}
	
		modellerStack.pointFixed = false;
		updateInfoPanel();
		repaint();
	}
	
	public void mouseDragged(MouseEvent e)
	{
//		System.out.println("mouseDragged");
		
		modellerStack.setMouseFlags(true, true, false, false, false, false);
		
		// Setting the xyz-positions of the mouse-cursor
		if(panelID==0) {
			modellerStack.xDragged = e.getX();
			modellerStack.yDragged = e.getY();
		} else if(panelID==1) {
			modellerStack.zDragged = e.getX();
			modellerStack.yDragged = e.getY();
		} else if(panelID==2) {
			modellerStack.xDragged = e.getX();
			modellerStack.zDragged = e.getY();
		}
		
		updateInfoPanel();
		statusPanel.updatePanel();
//		if(modellerStack.drawMode)	// refreshing, if the 3D-Mode is aktive
			refreshOtherDrawpanels();	
		repaint();
	}

	public void mouseMoved(MouseEvent e)
	{
//		System.out.println("mouseMoved in panel "+panelID);
		
		modellerStack.setMouseFlags(false, false, false, false, false, true);
		
		modellerStack.currentDrawPanelID = panelID;
		modellerStack.globalScale = scale;
		
		// Setting the xyz-positions of the mouse-cursor
		if(panelID==0) {
			modellerStack.xMoved = e.getX();
			modellerStack.yMoved = e.getY();
		} else if(panelID==1) {
			modellerStack.zMoved = e.getX();
			modellerStack.yMoved = e.getY();
		} else if(panelID==2) {
			modellerStack.xMoved = e.getX();
			modellerStack.zMoved = e.getY();
		}
		
		updateInfoPanel();
//		if(modellerStack.drawMode)	// refreshing, if the 3D-Mode is aktive
			refreshOtherDrawpanels();	
			
		repaint();
	}

	// Eventhandler for the drawPanel 
	public void componentHidden(ComponentEvent e)
	{
//		System.out.println("componentHidden");
	}

	public void componentMoved(ComponentEvent e)
	{
//		panelClipX		= e.getComponent().getBounds().getX();
//		panelClipY		= e.getComponent().getBounds().getY();
		panelClipX		= getBounds().getX();
		panelClipY		= getBounds().getY();
		panelClipWidth	= scrollPane.getWidth();
		panelClipHeight	= scrollPane.getHeight();
		
//		System.out.println(panelID+" (componentMoved: DP) "+panelClipX+" - "+panelClipY);
		
		repaint();
	}

	public void componentResized(ComponentEvent e)
	{
//		System.out.println("(DrawPanel) componentResized");
//
//		panelClipX		= e.getComponent().getBounds().getX();
//		panelClipY		= e.getComponent().getBounds().getY();
		panelClipX		= getBounds().getX();
		panelClipY		= getBounds().getY();
		panelClipWidth	= scrollPane.getWidth();
		panelClipHeight	= scrollPane.getHeight();
		
//		System.out.println("(componentResized) panelID: "+panelID+" - "+this.getWidth()+" - "+this.getHeight());
//		System.out.println("(scrollpane) panelID: "+panelID+" - "+panelClipWidth+" - "+panelClipHeight);
		
//		modellerStack.repaintDrawPanels();
	}

	public void componentShown(ComponentEvent e)
	{
//		System.out.println("componentShown");
	}

	public ViewPopup getViewPopup() {
		return viewPopup;
	}

	public void setViewPopup(ViewPopup viewPopup) {
		this.viewPopup = viewPopup;
	}

	public StatusPanel getStatusPanel() {
		return statusPanel;
	}

	public boolean isFetchButtonReleased() {
		return fetchButtonReleased;
	}

	public void setFetchButtonReleased(boolean fetchButtonReleased) {
		this.fetchButtonReleased = fetchButtonReleased;
	}

	public void setScrollPane(JScrollPane scrollPane) {
		this.scrollPane = scrollPane;
		scrollPane.addComponentListener(this);
	}

	public JScrollPane getScrollPane() {
		return scrollPane;
	}

	public Color getPanelBackground() {
		return panelBackground;
	}

	public void setPanelBackground(Color panelBackground) {
		this.panelBackground = panelBackground;
	}

	public void setPanelClipHeight(double panelClipHeight) {
		this.panelClipHeight = panelClipHeight;
	}

	public void setPanelClipWidth(double panelClipWidth) {
		this.panelClipWidth = panelClipWidth;
	}
}
