/*
Class-name: ModellerPanel.java, part of the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/

package modeller;

import interfaces.Layer;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import modeller.layer.LayerFrame;
import modeller.layer.LayerPanel;
import modeller.objects.Objects;
import modeller.popups.ViewPopup;
import modeller.preview3d.HelloUniverse;
import modeller.preview3d.J3dPreview;
import modeller.toolBars.GeometricsBar;
import modeller.toolBars.ToolsBar;
import modeller.toolBars.ViewBar;

public class ModellerPanel extends JPanel implements ComponentListener
{
	private ModellerStack modellerStack;	// Controller-Class for managing the mouse-interaction
	
	private JTabbedPane layerPane;
	private Objects objects;
	private StatusPanel statusPanel;
	private JPanel drawPanelContainer;
	private JScrollPane scrollPane[] = new JScrollPane[4]; //, scrollPane1, scrollPane2, scrollPane3; 	// DrawPanel
	private JPanel Preview3D_Panel;
	private ViewBar viewBar;
	private ViewPopup viewPopup;
	private ToolsBar toolsBar;
	private GeometricsBar geometricsBar;
	private boolean mode = true;
	
	public ModellerPanel(ModellerStack modellerStack)
	{
		this.modellerStack = modellerStack;
		
		modellerStack.modellerPanel = this;
		
		this.setLayout(new BorderLayout());
		
		initPanel();
	}
	
	/**
	 * Initializing the panels for the painting-tools, the draw-area and the layer-area
	 *
	 */
	private void initPanel()
	{
		Box hBox = Box.createHorizontalBox();
		
		toolsBar = new ToolsBar(modellerStack);
		geometricsBar = new GeometricsBar(modellerStack);

		drawPanelContainer = new JPanel();
		drawPanelContainer.setLayout(new GridLayout(2,2));
		modellerStack.drawPanelContainer = drawPanelContainer;

		statusPanel = new StatusPanel(modellerStack);
		layerPane 	= new JTabbedPane();

		modellerStack.layers.add(new Layer());
		modellerStack.layerPanel[0] = new LayerPanel(modellerStack, false, 0);	// Layerpanel beside drawpanel
		modellerStack.layerPanel[1] = new LayerPanel(modellerStack, true, 1);	// Layerpanel in EditLayerFrame
		modellerStack.layerFrame = new LayerFrame(modellerStack, modellerStack.layerPanel[1]);

		objects = new Objects(modellerStack);
		
		viewBar = new ViewBar(modellerStack);
		viewPopup = new ViewPopup(viewBar); // s, modellerStack);

		hBox.add(viewBar);
		hBox.add(toolsBar);
		hBox.add(geometricsBar);
		
		modellerStack.objects = objects;

		// Panel with the xy-view
		modellerStack.drawPanels[0] = new DrawPanel(statusPanel, modellerStack, 0);
		modellerStack.drawPanels[0].setViewPopup(viewPopup);
		scrollPane[0] = new JScrollPane(modellerStack.drawPanels[0]);	// DrawPanel
		scrollPane[0].addComponentListener(this);
		modellerStack.drawPanels[0].setScrollPane(scrollPane[0]);
//		actionController.drawPanels[0].setBorder(BorderFactory.createTitledBorder("Front-View "));
		
		// Panel with the yz-view
		modellerStack.drawPanels[1] = new DrawPanel(statusPanel, modellerStack, 1);
		modellerStack.drawPanels[1].setViewPopup(viewPopup);
		scrollPane[1] = new JScrollPane(modellerStack.drawPanels[1]);
		scrollPane[1].addComponentListener(this);
		modellerStack.drawPanels[1].setScrollPane(scrollPane[1]);
//		actionController.drawPanels[1].setBorder(BorderFactory.createTitledBorder("Side-View "));
		
		// Panel with the xz-view
		modellerStack.drawPanels[2] = new DrawPanel(statusPanel, modellerStack, 2);
		modellerStack.drawPanels[2].setViewPopup(viewPopup);
		scrollPane[2] = new JScrollPane(modellerStack.drawPanels[2]);
		scrollPane[2].addComponentListener(this);
		modellerStack.drawPanels[2].setScrollPane(scrollPane[2]);
//		actionController.drawPanels[2].setBorder(BorderFactory.createTitledBorder("Top-View "));

		// Panel with the Java3d-preview (comming soon)
		Preview3D_Panel = new JPanel();
		JButton swapBtn = new JButton("3D-Preview interface ... coming soon!");
		swapBtn.setBackground(Color.WHITE);
		Preview3D_Panel.setLayout(new GridLayout(1, 1));
//		Preview3D_Panel.add(swapBtn);
		
		J3dPreview j3dPreview = new J3dPreview(this);
		j3dPreview.setBoundaries(modellerStack.boundaryWidth,
								 modellerStack.boundaryHeight,
								 modellerStack.boundaryDepth,
								 modellerStack.rasterWidth,
								 modellerStack.pixelPerMm);
		j3dPreview.setObjectData(modellerStack.cadObjects, modellerStack.layers);
		Preview3D_Panel.add(j3dPreview);
		
//		Preview3D_Panel.add(new HelloUniverse());
		scrollPane[3] = new JScrollPane(Preview3D_Panel);
		scrollPane[3].addComponentListener(this);
		

//		JToolBar swapToolBar = new JToolBar();
//		swapToolBar.add(layerPanel);
//		layerPanel.setLayout(new GridLayout(1,1));
		
		updateDrawPanelContainer(0, mode);
		
		layerPane = new JTabbedPane();
		layerPane.add("Layer", modellerStack.layerPanel[0]);
		layerPane.add("Layer-Properties", modellerStack.editLayerProperties[0]);
		
		layerPane.setPreferredSize(new Dimension(310, 100));
		layerPane.setMinimumSize(new Dimension(310, 100));
		layerPane.setMaximumSize(new Dimension(310, 100));
		
		this.add(hBox, BorderLayout.NORTH);
		this.add(objects, BorderLayout.WEST);
		this.add(drawPanelContainer, BorderLayout.CENTER);	// Container for xy- yz- xz and Java3d-view
		this.add(layerPane, BorderLayout.EAST);
		this.add(statusPanel, BorderLayout.SOUTH);
	}

	public void updateDrawPanelContainer(int panelID, boolean mode)
	{
//		System.out.println("componentResized");
		System.out.println("drawPanelContainer.getWidth(): "+drawPanelContainer.getWidth()+" - drawPanelContainer.getHeight(): "+drawPanelContainer.getHeight());
		
		drawPanelContainer.setVisible(false);
		drawPanelContainer.removeAll();
		
		if(mode) {
			drawPanelContainer.setLayout(new GridLayout(2,2));
			drawPanelContainer.add(scrollPane[0]); //new JScrollPane(modellerStack.drawPanels[0]));
			drawPanelContainer.add(scrollPane[1]); //new JScrollPane(modellerStack.drawPanels[1]));
			drawPanelContainer.add(scrollPane[2]); //new JScrollPane(modellerStack.drawPanels[2]));
			drawPanelContainer.add(Preview3D_Panel); // new JScrollPane(Preview3D_Panel));
		}
		else if(panelID==3) {
			drawPanelContainer.setLayout(new GridLayout(1,1));
			drawPanelContainer.add(Preview3D_Panel);

		}
		else {
			drawPanelContainer.setLayout(new GridLayout(1,1));
			drawPanelContainer.add(scrollPane[panelID]);
		}
		drawPanelContainer.setVisible(true);
	}
	
//	public JPanel getDrawPanelContainer() {
//		return drawPanelContainer;
//	}

//	public JPanel getPreview3D_Panel() {
//		return Preview3D_Panel;
//	}

	public void componentResized(ComponentEvent e)
	{
		// TODO Auto-generated method stub
//		if(e.getSource() instanceof JScrollPane) {
			System.out.println("componentResized: JScrollPane");
			
//			int id = ((DrawPanelScrollPane)e.getSource()).getPanelID();
//			modellerStack.drawPanels[id].getScrollPane().componentResized(e);
//		}

			int w, h;
			
			w = e.getComponent().getWidth();
			h = e.getComponent().getHeight();
			
			for(int i=0; i<3; i++) {
				modellerStack.drawPanels[i].setPanelClipWidth(w);
				modellerStack.drawPanels[i].setPanelClipHeight(h);
			}

			
//		System.out.println(" - componentResized: "+e.getComponent().getWidth()+" - "+e.getComponent().getHeight());
		
	}

	public void componentMoved(ComponentEvent e)
	{
//		if(e.getSource() instanceof JScrollPane) {
			System.out.println("componentMoved: JScrollPane");

			int w, h;
			
			w = e.getComponent().getWidth();
			h = e.getComponent().getHeight();
			
			for(int i=0; i<3; i++) {
				modellerStack.drawPanels[i].setPanelClipWidth(w);
				modellerStack.drawPanels[i].setPanelClipHeight(h);
			}
//		}
//			modellerStack.repaintDrawPanels();
	}

	public void componentShown(ComponentEvent e) {
		System.out.println("componentShown: JScrollPane"+e.getComponent().getWidth()+" - "+e.getComponent().getHeight());
		
		int w, h;
		
		w = e.getComponent().getWidth();
		h = e.getComponent().getHeight();
		
		for(int i=0; i<3; i++) {
			modellerStack.drawPanels[i].setPanelClipWidth(w);
			modellerStack.drawPanels[i].setPanelClipHeight(h);
		}

//		modellerStack.repaintDrawPanels();
	}

	public void componentHidden(ComponentEvent e) {
//		System.out.println("componentHidden: JScrollPane");
		
	}

	public ToolsBar getToolsBar() {
		return toolsBar;
	}

	public GeometricsBar getGeometricsBar() {
		return geometricsBar;
	}

	public ViewBar getViewBar() {
		return viewBar;
	}

	public JTabbedPane getLayerPane() {
		return layerPane;
	}
}
