/*
Class-name: StaticDrawing.java, part of the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a (soon) integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/
package modeller.draw;

import interfaces.CadObject;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

public class StaticDrawing
{
//	 Constant stroke-objects
	public static final BasicStroke BASICSTROKE = new BasicStroke(1f);
	public static final BasicStroke FINE_DOTTED_STROKE = new BasicStroke( 1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 4, new float[]{ 2.5f, 5f }, 0);
	public static final BasicStroke DASHED_STROKE = new BasicStroke( 1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 4, new float[]{ 6f, 6f }, 0);
	
	// Constants for the extruding-directions
	public static final int EXTRUDING_X = 0;	// Extruding in X-Direction
	public static final int EXTRUDING_Y = 1;	// Extruding in Y-Direction
	public static final int EXTRUDING_Z = 2;	// Extruding in Z-Direction (default)
	
	// Constants for 2D-objects
	public static final int GEOMETRY_2D_LINE = 0;			// Geometrytyp is a simple line (default)
	public static final int GEOMETRY_2D_POLYGON = 1;		// Geometrytyp is a poly-line
	public static final int GEOMETRY_2D_CLOSED_POLYGON = 2;	// Geometrytyp is a closed poly-line
	public static final int GEOMETRY_2D_CIRCLE = 3;			// Geometrytyp is a circle
	public static final int GEOMETRY_2D_ELLIPSE = 4;		// Geometrytyp is a ellipse
	public static final int GEOMETRY_2D_TRIANGLE = 5;		// Geometrytyp is a triangle
	public static final int GEOMETRY_2D_SHAPE = 6;			// Geometrytyp is a shape
	
	// Constants for 3D-objects
	public static final int GEOMETRY_3D_SPHERE = 7;			// Geometrytyp is a sphere
	public static final int GEOMETRY_3D_CUBE = 8;			// Geometrytyp is a cube
	public static final int GEOMETRY_3D_BOX =9;				// Geometrytyp is a box
	public static final int GEOMETRY_3D_CYLINDER = 10;		// Geometrytyp is a cylinder
	public static final int GEOMETRY_3D_CONE = 11;			// Geometrytyp is a cone	
	
	public static Shape swapShape = null;
	
	public static BasicStroke GET_DASHED_STROKE(double scale)
	{
		return 	new BasicStroke( (float)(1d / scale), BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 4, new float[]{ 6f, 6f }, 0);
	}

	public static BasicStroke GET_BASICSTROKE(double scale)
	{
		return 	new BasicStroke((float)(1d / scale));
	}

	/**
	 * Paints the selected CadObject object
	 * @param g2
	 */
	public static void paintingObject(Graphics2D g2, CadObject swapCadObject, int selectedObjectIndex, int panelID, double scale, boolean selected)
	{
		swapShape = swapCadObject.getShape(panelID);
		swapCadObject.setRendered2D(true);

		if(swapCadObject.isFilled()) {					// if object is filled, paint filled oject
			if(swapCadObject.getTransparency()<1d && swapCadObject.getTransparency()>0d)
				g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, (float)swapCadObject.getTransparency()));
			if(swapCadObject.getTransparency()>0d) {
				g2.setColor(swapCadObject.getFillColor());
				g2.fill(swapShape);
			}
		}
		
		// Resetting the transparency
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1f));

		// painting dashed stroke if object is selected
		if(selected) { // modellerStack.selectedObject==selectedObjectIndex) {
			g2.setStroke((StaticDrawing.GET_DASHED_STROKE(scale)));
			g2.setColor(Color.DARK_GRAY);
			g2.draw(StaticDrawing.drawMarkingBox(swapShape));
			g2.setStroke((StaticDrawing.GET_BASICSTROKE(scale)));

			// painting the filling of the marking-spots 
			g2.setColor(Color.CYAN);
			g2.fill(StaticDrawing.drawMarkingSpots(swapShape, 1d / scale, 3));

			// painting the border of the marking-spots 
			g2.setColor(Color.BLACK);
			g2.draw(StaticDrawing.drawMarkingSpots(swapShape, 1d / scale, 3));
			g2.setStroke(new BasicStroke((float)(swapCadObject.getLineWidth())));
		}
		
		// painting the outline of the object
		g2.setColor(swapCadObject.getLineColor());
		g2.draw(swapShape);
		
		// Resetting the translation, if it's necessary
//		if(translateX < 0 || translateY < 0 || translateZ < 0)
//			g2.translate(-translX, -translY);
	}
	
	/**
	 * Setting the rendered-flag for all CAD-objects
	 * @param cadObjects
	 * @param rendered
	 */
	public static void setAllCadObjectsRendered(ArrayList<CadObject> cadObjects, boolean rendered)
	{
		for(int i=0; i<cadObjects.size(); i++)
			cadObjects.get(i).setRendered2D(rendered);			
	}

	/**
	 * Drawing the cursor-cross
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 * @param scale
	 * @return
	 */
	public static Shape drawCross(double x, double y, double z, int panelID, int width, int height)
	{
		GeneralPath path = new GeneralPath();
		
		float xPos = 0, yPos = 0;
		
		if(panelID==0) {
			yPos = (float)y;
			xPos = (float)x;
		} else if(panelID==1) {
			yPos = (float)y;
			xPos = (float)z;
		} else if(panelID==2 ) {
			yPos = (float)z;
			xPos = (float)x;
		}  
		
		path.moveTo(0f , yPos);
		path.lineTo(width, yPos);

		path.moveTo(xPos, 0f);
		path.lineTo(xPos, height);

		return path;
	}
	
	/**
	 * Drawing the help-line
	 * @param x1
	 * @param y1
	 * @param z1
	 * @param x2
	 * @param y2
	 * @param z2
	 * @param panelID
	 * @param width
	 * @param height
	 * @param scale
	 * @return
	 */
	public static Shape drawPaintLine(int x1, int y1, int z1, int x2, int y2, int z2, int panelID, int width, int height)
	{
		double yStart = 0, yStop = 0, x2Subx1, z2Subz1;
		
		if(panelID==0) {
			if(x2==x1)		// Catching the division by zero
				x2Subx1 = 1;
			else
				x2Subx1 = x2 - x1;
			
			yStart = y1 - ( x1 * ( (y2-y1)/x2Subx1 ) );
			yStop  = y1 + ( ( (y2-y1)/x2Subx1 ) * ( width - x1) );
			
		} else if(panelID==1) {
			if(z2==z1)		// Catching the division by zero
				z2Subz1 = 1;
			else
				z2Subz1 = z2 - z1;
			
			yStart = y1 - ( z1 * ( (y2-y1)/z2Subz1 ) );
			yStop  = y1 + ( ( (y2-y1)/z2Subz1 ) * ( width - z1) );
			
		} else if(panelID==2) {
			if(x2==x1)		// Catching the division by zero
				x2Subx1 = 1;
			else
				x2Subx1 = x2 - x1;
			
			yStart = z1 - ( x1 * ( (z2-z1)/x2Subx1 ) );
			yStop  = z1 + ( ( (z2-z1)/x2Subx1 ) * ( width - x1) );
		}

		return new Line2D.Double(0, yStart, width, yStop);
	}

	/**
	 * 	Returning a Shape for a (maybe dashed) box, as a frame around of the object-shape 
	 * @param shape
	 * @return
	 */
	public static Shape drawMarkingBox(Shape shape)
	{		
		return new Rectangle2D.Double(shape.getBounds2D().getMinX(), shape.getBounds2D().getMinY(), shape.getBounds2D().getWidth(), shape.getBounds2D().getHeight());
	}
	
	/**
	 * Return a a simple BasicStroke
	 * @param scale
	 * @return
	 */
	public static BasicStroke buildStroke(float scale)
	{
		BasicStroke basicStroke = new BasicStroke(2f/scale); // g2.setStroke(basicStroke); raster.generateRaster(getWidth(), getHeight(), scale); g2.drawImage(raster.getRasterImage(), 0, 0, null);
		
		return basicStroke;
	}
	
	/**
	 * Returning a Shape with a single marking-spot
	 * @param x
	 * @param y
	 * @param scale
	 * @param size
	 * @return
	 */
	public static Shape drawMarkingSpot(double x, double y, double scale, int size)
	{
		double spotSize = size / scale;
		
		return new Rectangle2D.Double(x - spotSize, y - spotSize, 2*spotSize , 2*spotSize);
	}
	
	/**
	 * Returning a Shape, for drawing marking-spots on the coner of the object-shape 
	 * @param shape
	 * @param scale
	 * @param size
	 * @return
	 */
	public static Shape drawMarkingSpots(Shape shape, double scale, int size)
	{
		float spotSize = (float)(size*scale); // size / scale;
		
		float x, y;
		
		GeneralPath gp = new GeneralPath();

		Rectangle2D bounds = shape.getBounds2D(); 
		
		x = (float)bounds.getMinX(); y = (float)bounds.getMinY();
		gp.moveTo(x-spotSize, y-spotSize); gp.lineTo(x+spotSize, y-spotSize); gp.lineTo(x+spotSize, y+spotSize); gp.lineTo(x-spotSize, y+spotSize); gp.closePath();

		x = (float)bounds.getMaxX(); y = (float)bounds.getMinY();
		gp.moveTo(x-spotSize, y-spotSize); gp.lineTo(x+spotSize, y-spotSize); gp.lineTo(x+spotSize, y+spotSize); gp.lineTo(x-spotSize, y+spotSize); gp.closePath();

		x = (float)bounds.getMinX(); y = (float)bounds.getMaxY();
		gp.moveTo(x-spotSize, y-spotSize); gp.lineTo(x+spotSize, y-spotSize); gp.lineTo(x+spotSize, y+spotSize); gp.lineTo(x-spotSize, y+spotSize); gp.closePath();

		x = (float)bounds.getMaxX(); y = (float)bounds.getMaxY();
		gp.moveTo(x-spotSize, y-spotSize); gp.lineTo(x+spotSize, y-spotSize); gp.lineTo(x+spotSize, y+spotSize); gp.lineTo(x-spotSize, y+spotSize); gp.closePath();

		return gp;
	}

	public static Shape drawMarkingSpot(double x, double y, double z, double scale, int size, int panelID)
	{
		if(panelID==0)
			return new Rectangle2D.Double(x*scale - size, y*scale - size, 2*size, 2*size);
		else if(panelID==1)
			return new Rectangle2D.Double(z*scale - size, y*scale - size, 2*size, 2*size);
		else // if(panelID==2)
			return new Rectangle2D.Double(x*scale - size, z*scale - size, 2*size, 2*size);
	}	
}
