/*
Class-name: ObjectsPanel.java, part of the 'Open-Simulation-Platform'

The 'Open-Simulation-Platform' provides a dynamically GUI, with a plugable 2D/3D CAD-interface and a (soon) integrated plugable simulation-interface 

Copyright (C) 2006 Karsten Bettray

This program is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with this library;
if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA *
*/
package modeller.objects;

import interfaces.CadObject;
import interfaces.Layer;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import modeller.ModellerStack;
import modeller.popups.ObjectsPopup;

public class ObjectsPanel extends JPanel implements MouseListener, TreeSelectionListener 
{
	private ModellerStack modellerStack;
	private JTree tree;
	private ObjectsPopup objPopup;
	
	private Vector<MyDefaultMutableTreeNode> layerNodes = new Vector<MyDefaultMutableTreeNode>();
	
	public ObjectsPanel(ModellerStack modellerStack)
	{
		this.modellerStack = modellerStack;
		
		modellerStack.objectsPanel = this;
		
		objPopup = new ObjectsPopup(modellerStack);
		
		this.setLayout(new GridLayout(1, 1));
		this.setBackground(Color.WHITE);
		this.setBorder(BorderFactory.createTitledBorder("Objects "));
		
		updateObjectTree();
	}

	/**
	 * Updating (new generating) the JTree, with the layers and objects
	 *
	 */
	public void updateObjectTree()
	{
		this.removeAll();
		
		DefaultMutableTreeNode rootNode;
		MyDefaultMutableTreeNode layerNode, objectChild;	// layerNode
		Layer layer;
		CadObject cadObject;
		int row = 0;

		layerNodes.clear();
		
		rootNode = new DefaultMutableTreeNode("Layerbased objects");
		
		for(int j=0; j < modellerStack.layers.size(); j++) {
			layer = modellerStack.layers.get(j);
			layerNode = new MyDefaultMutableTreeNode(layer.getName());
			layerNode.setId(layer.getLayerID());
			layerNode.setTyp(MyDefaultMutableTreeNode.LAYER_NODE);
			layerNode.setRow(row++);
			layerNodes.add(layerNode);
			for(int i=0; i < modellerStack.cadObjects.size(); i++) {
				if(modellerStack.cadObjects.get(i).getLayerID()==j) {
					row++;
					cadObject = modellerStack.cadObjects.get(i);
					objectChild = new MyDefaultMutableTreeNode(cadObject.getObjectId()+": "+cadObject.getName());
					objectChild.setId(cadObject.getObjectId());
					objectChild.setTyp(MyDefaultMutableTreeNode.OBJECT_NODE);
					layerNode.add(objectChild);
				}
			}
			rootNode.add(layerNode);			
		}
		
		tree = new JTree(rootNode);
		tree.setBackground(getBackground());
		TreeSelectionModel tsm = new DefaultTreeSelectionModel();
		tsm.setSelectionMode(TreeSelectionModel.CONTIGUOUS_TREE_SELECTION); //SINGLE_TREE_SELECTION);
		tree.setSelectionModel(tsm);
		tree.addMouseListener(this);
		tree.addTreeSelectionListener(this);
		
		tree.setRootVisible(true);

		this.add(tree);
		this.setVisible(false); this.setVisible(true);

		for(int i=0; i<layerNodes.size(); i++) {
			if(layerNodes.get(i).getTyp()==MyDefaultMutableTreeNode.LAYER_NODE) {
				if(modellerStack.layers.get(i).isFolderOpen())
					tree.expandRow(layerNodes.get(i).getRow()+1);
			}
		}
	}
	
	private void treeEvent(Object object)
	{
//		Object object = e.getSource();
		
//		System.out.println(object.getClass().getCanonicalName());
		
		if(object instanceof MouseEvent)
			objPopup.show(null, ((MouseEvent)object).getX(), ((MouseEvent)object).getY());

		if(object instanceof JTree) {
			JTree tr = (JTree)object;
			TreePath tp = tr.getSelectionPath();

			// Switching to the selected FemObject
			if(tp!=null && tp.getLastPathComponent() instanceof MyDefaultMutableTreeNode) {

				MyDefaultMutableTreeNode treeNode = (MyDefaultMutableTreeNode)tp.getLastPathComponent();
				
				// If node is a object-node, update the properties-panel and if necessary, update the layer-panel
				if(treeNode.getTyp()==MyDefaultMutableTreeNode.OBJECT_NODE) {

					modellerStack.selectedObject=treeNode.getId();
					modellerStack.repaintDrawPanels();
					modellerStack.objectsProperties.updateObjectList(modellerStack.cadObjects.get(treeNode.getId()), true);
					modellerStack.layerPanel[0].updateLayer(modellerStack.cadObjects.get(treeNode.getId()).getLayerID());
					modellerStack.layerPanel[1].updateLayer(modellerStack.cadObjects.get(treeNode.getId()).getLayerID());
				}
				else	// If node is a layer-node, set the folding-flag in layer-object
					if(treeNode.getTyp()==MyDefaultMutableTreeNode.LAYER_NODE) {

					modellerStack.layers.get(treeNode.getId()).setFolderOpen(tree.isExpanded(tp));
					
					modellerStack.layerPanel[0].updateLayer(treeNode.getId());
					modellerStack.layerPanel[1].updateLayer(treeNode.getId());

					// Setting bool-flag, for the folding-state
					modellerStack.layers.get(treeNode.getId()).setFolderOpen(tree.isExpanded(tp));
				}
			}
		}
	}
	
	public void valueChanged(TreeSelectionEvent e)
	{
		treeEvent(e.getSource());	
	}

	public void mouseClicked(MouseEvent e)
	{
//		treeEvent(e.getSource());
	}
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
	public void mousePressed(MouseEvent e) {
		 	
		if(e.getButton()==MouseEvent.BUTTON3) {
			TreePath tp = tree.getClosestPathForLocation(e.getX(), e.getY());
			
			MyDefaultMutableTreeNode treeNode = (MyDefaultMutableTreeNode)tp.getLastPathComponent();

			System.out.println("Treenode: "+treeNode.getId());
			
			objPopup.setObjID(treeNode.getId());
			objPopup.show(e.getComponent(), e.getX(), e.getY());
		}
	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	public class MyDefaultMutableTreeNode extends DefaultMutableTreeNode
	{
		public static final int OBJECT_NODE = 0;
		public static final int LAYER_NODE = 1;
		
		private String nodeName = null;
		private int id = 0;
		private int typ = 0;
		private boolean folderOpen = true;
		private int row = 0;

		public MyDefaultMutableTreeNode(String string)
		{
			super(string);

			this.nodeName = string;
		}
		
		public int getId() {
			return id;
		}
		public void setId(int id) {
			this.id = id;
		}

		public int getTyp() {
			return typ;
		}
		public void setTyp(int typ) {
			this.typ = typ;
		}

		public boolean isFolderOpen() {
			return folderOpen;
		}

		public void setFolderOpen(boolean folderOpen) {
			this.folderOpen = folderOpen;
		}

		public String getNodeName() {
			return nodeName;
		}

		public void setNodeName(String nodeName) {
			this.nodeName = nodeName;
		}

		public int getRow() {
			return row;
		}

		public void setRow(int row) {
			this.row = row;
		}
	}
}
